@@ -16,11 +16,12 @@ struct _RfLVNCServer {
 	char *desktop_name;
 	unsigned int width;
 	unsigned int height;
+	bool resize;
 	bool running;
 };
 G_DEFINE_TYPE(RfLVNCServer, rf_lvnc_server, RF_TYPE_VNC_SERVER)
 
-static int _on_socket_in(GSocket *socket, GIOCondition condition, void *data)
+static int on_socket_in(GSocket *socket, GIOCondition condition, void *data)
 {
 	RfLVNCServer *this = data;
 
@@ -33,23 +34,25 @@ static int _on_socket_in(GSocket *socket, GIOCondition condition, void *data)
 	return G_SOURCE_CONTINUE;
 }
 
-static void _on_client_gone(rfbClientRec *client)
+static void on_client_gone(rfbClientRec *client)
 {
 	RfLVNCServer *this = client->screen->screenData;
+	RfVNCServer *super = RF_VNC_SERVER(this);
 	GSource *source = client->clientData;
+
 	g_source_destroy(source);
 	g_source_unref(source);
 	if (this->clients-- == 1)
-		rf_vnc_server_handle_last_client(RF_VNC_SERVER(this));
+		rf_vnc_server_handle_last_client(super);
 }
 
-static enum rfbNewClientAction _on_new_client(rfbClientRec *client)
+static enum rfbNewClientAction on_new_client(rfbClientRec *client)
 {
-	client->clientGoneHook = _on_client_gone;
+	client->clientGoneHook = on_client_gone;
 	return RFB_CLIENT_ACCEPT;
 }
 
-static int _on_set_desktop_size(
+static int on_set_desktop_size(
 	int width,
 	int height,
 	int num_screens,
@@ -58,36 +61,46 @@ static int _on_set_desktop_size(
 )
 {
 	RfLVNCServer *this = client->screen->screenData;
+	RfVNCServer *super = RF_VNC_SERVER(this);
+
+	if (!this->resize)
+		return rfbExtDesktopSize_ResizeProhibited;
+
 	if (width != this->width || height != this->height)
-		rf_vnc_server_handle_resize_event(
-			RF_VNC_SERVER(this), width, height
-		);
+		rf_vnc_server_handle_resize_event(super, width, height);
+
 	return rfbExtDesktopSize_Success;
 }
 
 static void
-_on_keysym_event(rfbBool direction, rfbKeySym keysym, rfbClientRec *client)
+on_keysym_event(rfbBool direction, rfbKeySym keysym, rfbClientRec *client)
 {
 	RfLVNCServer *this = client->screen->screenData;
+	RfVNCServer *super = RF_VNC_SERVER(this);
 	bool down = direction != 0;
-	rf_vnc_server_handle_keysym_event(RF_VNC_SERVER(this), keysym, down);
+
+	rf_vnc_server_handle_keysym_event(super, keysym, down);
 }
 
-static void _on_pointer_event(int mask, int x, int y, rfbClientRec *client)
+static void on_pointer_event(int mask, int x, int y, rfbClientRec *client)
 {
 	RfLVNCServer *this = client->screen->screenData;
+	RfVNCServer *super = RF_VNC_SERVER(this);
 	const double rx = (double)x / this->width;
 	const double ry = (double)y / this->height;
-	rf_vnc_server_handle_pointer_event(RF_VNC_SERVER(this), rx, ry, mask);
+
+	rf_vnc_server_handle_pointer_event(super, rx, ry, mask);
 }
 
-static void _on_clipboard_text(char *text, int length, rfbClientRec *client)
+static void on_clipboard_text(char *text, int length, rfbClientRec *client)
 {
 	RfLVNCServer *this = client->screen->screenData;
-	rf_vnc_server_handle_clipboard_text(RF_VNC_SERVER(this), text);
+	RfVNCServer *super = RF_VNC_SERVER(this);
+
+	rf_vnc_server_handle_clipboard_text(super, text);
 }
 
-static int _on_incoming(
+static int on_incoming(
 	GSocketService *service,
 	GSocketConnection *connection,
 	GObject *source_object,
@@ -95,6 +108,7 @@ static int _on_incoming(
 )
 {
 	RfLVNCServer *this = data;
+	RfVNCServer *super = RF_VNC_SERVER(this);
 
 	if (this->screen == NULL) {
 		this->screen = rfbGetScreen(
@@ -116,12 +130,12 @@ static int _on_incoming(
 		this->screen->versionString = "ReFrame VNC Server";
 		this->screen->screenData = this;
 		this->screen->cursor = NULL;
-		this->screen->newClientHook = _on_new_client;
-		this->screen->setDesktopSizeHook = _on_set_desktop_size;
-		this->screen->ptrAddEvent = _on_pointer_event;
-		this->screen->kbdAddEvent = _on_keysym_event;
-		this->screen->setXCutText = _on_clipboard_text;
-		this->screen->setXCutTextUTF8 = _on_clipboard_text;
+		this->screen->newClientHook = on_new_client;
+		this->screen->setDesktopSizeHook = on_set_desktop_size;
+		this->screen->ptrAddEvent = on_pointer_event;
+		this->screen->kbdAddEvent = on_keysym_event;
+		this->screen->setXCutText = on_clipboard_text;
+		this->screen->setXCutTextUTF8 = on_clipboard_text;
 		if (this->passwords[0] != NULL &&
 		    this->passwords[0][0] != '\0') {
 			this->screen->authPasswdData = this->passwords;
@@ -132,11 +146,11 @@ static int _on_incoming(
 	GSocket *socket = g_socket_connection_get_socket(connection);
 	g_debug("VNC: Got new connection %p.", socket);
 	if (++this->clients == 1)
-		rf_vnc_server_handle_first_client(RF_VNC_SERVER(this));
+		rf_vnc_server_handle_first_client(super);
 	// Don't attach source on new client hook, because it may be called
 	// before we set client data.
 	GSource *source = g_socket_create_source(socket, this->io_flags, NULL);
-	g_source_set_callback(source, G_SOURCE_FUNC(_on_socket_in), this, NULL);
+	g_source_set_callback(source, G_SOURCE_FUNC(on_socket_in), this, NULL);
 	g_source_attach(source, NULL);
 	const int fd = g_socket_get_fd(socket);
 	// `rfbClient` owns fd, but we got it from `GSocketConnection`.
@@ -144,23 +158,24 @@ static int _on_incoming(
 	client->clientData = source;
 	// Just in case client disconnects very soon.
 	if (client->sock == -1) {
-		_on_client_gone(client);
+		on_client_gone(client);
 		return false;
 	}
 
 	return true;
 }
 
-static void _dispose(GObject *o)
+static void dispose(GObject *o)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(o);
+	RfVNCServer *super = RF_VNC_SERVER(this);
 
-	rf_vnc_server_stop(RF_VNC_SERVER(this));
+	rf_vnc_server_stop(super);
 
 	G_OBJECT_CLASS(rf_lvnc_server_parent_class)->dispose(o);
 }
 
-static void _finalize(GObject *o)
+static void finalize(GObject *o)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(o);
 
@@ -169,7 +184,7 @@ static void _finalize(GObject *o)
 	G_OBJECT_CLASS(rf_lvnc_server_parent_class)->finalize(o);
 }
 
-static void _start(RfVNCServer *super)
+static void start(RfVNCServer *super)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -194,6 +209,11 @@ static void _start(RfVNCServer *super)
 			this->height
 		);
 	}
+	this->resize = rf_config_get_resize(this->config);
+	g_message(
+		"VNC: Client resizing will be %s.",
+		this->resize ? "allowed" : "prohibited"
+	);
 
 	g_autoptr(GError) error = NULL;
 	g_autofree char *ip = rf_config_get_vnc_ip(this->config);
@@ -222,20 +242,20 @@ static void _start(RfVNCServer *super)
 			port,
 			error->message);
 	g_signal_connect(
-		this->service, "incoming", G_CALLBACK(_on_incoming), this
+		this->service, "incoming", G_CALLBACK(on_incoming), this
 	);
 
 	this->running = true;
 }
 
-static bool _is_running(RfVNCServer *super)
+static bool is_running(RfVNCServer *super)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
 	return this->running;
 }
 
-static void _stop(RfVNCServer *super)
+static void stop(RfVNCServer *super)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -257,7 +277,7 @@ static void _stop(RfVNCServer *super)
 	g_clear_pointer(&this->passwords[0], g_free);
 }
 
-static void _set_desktop_name(RfVNCServer *super, const char *desktop_name)
+static void set_desktop_name(RfVNCServer *super, const char *desktop_name)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -273,7 +293,7 @@ static void _set_desktop_name(RfVNCServer *super, const char *desktop_name)
 	// }
 }
 
-static void _send_clipboard_text(RfVNCServer *super, const char *text)
+static void send_clipboard_text(RfVNCServer *super, const char *text)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -301,11 +321,11 @@ static void _send_clipboard_text(RfVNCServer *super, const char *text)
 }
 
 static void
-_update(RfVNCServer *super,
-	GByteArray *buf,
-	unsigned int width,
-	unsigned int height,
-	const struct rf_rect *damage)
+update(RfVNCServer *super,
+       GByteArray *buf,
+       unsigned int width,
+       unsigned int height,
+       const struct rf_rect *damage)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -349,7 +369,7 @@ _update(RfVNCServer *super,
 	rfbProcessEvents(this->screen, 0);
 }
 
-static void _flush(RfVNCServer *super)
+static void flush(RfVNCServer *super)
 {
 	RfLVNCServer *this = RF_LVNC_SERVER(super);
 
@@ -370,16 +390,16 @@ static void rf_lvnc_server_class_init(RfLVNCServerClass *klass)
 	GObjectClass *o_class = G_OBJECT_CLASS(klass);
 	RfVNCServerClass *v_class = RF_VNC_SERVER_CLASS(klass);
 
-	o_class->dispose = _dispose;
-	o_class->finalize = _finalize;
+	o_class->dispose = dispose;
+	o_class->finalize = finalize;
 
-	v_class->start = _start;
-	v_class->is_running = _is_running;
-	v_class->stop = _stop;
-	v_class->set_desktop_name = _set_desktop_name;
-	v_class->send_clipboard_text = _send_clipboard_text;
-	v_class->update = _update;
-	v_class->flush = _flush;
+	v_class->start = start;
+	v_class->is_running = is_running;
+	v_class->stop = stop;
+	v_class->set_desktop_name = set_desktop_name;
+	v_class->send_clipboard_text = send_clipboard_text;
+	v_class->update = update;
+	v_class->flush = flush;
 }
 
 static void rf_lvnc_server_init(RfLVNCServer *this)
@@ -394,6 +414,7 @@ static void rf_lvnc_server_init(RfLVNCServer *this)
 	this->desktop_name = NULL;
 	this->width = 0;
 	this->height = 0;
+	this->resize = true;
 	this->running = false;
 }
 
